@@ -1,4 +1,6 @@
-(* Copyright (C) 2006, 2007 Julia Lawall
+(*
+ * Copyright (C) 2010, University of Copenhagen DIKU and INRIA.
+ * Copyright (C) 2006, 2007 Julia Lawall
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License (GPL)
@@ -48,7 +50,7 @@ let close_box _ = () in
 let force_newline _ = print_text "\n" in
 
 let start_block () = force_newline(); indent() in
-let end_block () = unindent(); force_newline () in
+let end_block () = unindent true; force_newline () in
 let print_string_box s = print_string s in
 
 let print_option = Common.do_option in
@@ -64,7 +66,7 @@ let outdent _ = () (* should go to leftmost col, does nothing now *) in
 
 let pretty_print_c =
   Pretty_print_c.mk_pretty_printers pr_celem pr_cspace
-    force_newline indent outdent unindent in
+    force_newline indent outdent (function _ -> unindent true) in
 
 (* --------------------------------------------------------------------- *)
 (* Only for make_hrule, print plus code, unbound metavariables *)
@@ -104,12 +106,14 @@ let print_around printer term = function
       print_anything bef; printer term; print_anything aft in
 
 let print_string_befaft fn fn1 x info =
+  let print ln col =
+    function Ast.Noindent s | Ast.Indent s -> print_string s ln col in
   List.iter
-    (function (s,ln,col) -> fn1(); print_string s ln col; force_newline())
+    (function (s,ln,col) -> fn1(); print ln col s; force_newline())
     info.Ast.strbef;
   fn x;
   List.iter
-    (function (s,ln,col) -> force_newline(); fn1(); print_string s ln col)
+    (function (s,ln,col) -> force_newline(); fn1(); print ln col s)
     info.Ast.straft in
 let print_meta (r,x) = print_text x in
 
@@ -133,10 +137,25 @@ let mcode fn (s,info,mc,pos) =
 	  (function line_before ->
 	    function (str,line,col) ->
 	      match line_before with
-		None -> print_string str line col; Some line
-	      |	Some lb when line =|= lb ->
+		None ->
+		  let str =
+		    match str with
+		      Ast.Noindent s -> unindent false; s
+		    | Ast.Indent s -> s in
 		  print_string str line col; Some line
-	      |	_ -> force_newline(); print_string str line col; Some line)
+	      |	Some lb when line =|= lb ->
+		  Printf.printf "some, line same case\n";
+		  let str = match str with Ast.Noindent s | Ast.Indent s -> s in
+		  print_string str line col; Some line
+	      |	_ ->
+		  force_newline();
+		  (* not super elegant to put side-effecting unindent in a let
+		     expression... *)
+		  let str =
+		    match str with
+		      Ast.Noindent s -> unindent false; s
+		    | Ast.Indent s -> s in
+		  print_string str line col; Some line)
 	  lb comments in
       let line_before = print_comments None info.Ast.strbef in
       (match line_before with
@@ -149,10 +168,6 @@ let mcode fn (s,info,mc,pos) =
 	 should really store parsed versions of the strings, but make a cheap
 	 effort here
          print_comments takes care of interior newlines *)
-      (match List.rev info.Ast.straft with
-	(str,_,_)::_ when String.length str > 0 && String.get str 0 = '#' ->
-	  force_newline()
-      |	_ -> ());
       ()
       (* printing for rule generation *)
   | (true, Ast.MINUS(_,_,_,plus_stream)) ->
@@ -197,7 +212,8 @@ let handle_metavar name fn =
 	(* call mcode to preserve the -+ annotation *)
 	mcode (fun _ _ _ -> fn e) name
       else fn e);
-      let rcol = if lcol = unknown then unknown else lcol + (String.length b) in
+      let rcol =
+	if lcol = unknown then unknown else lcol + (String.length b) in
       pr_barrier line rcol
 in
 (* --------------------------------------------------------------------- *)
@@ -309,19 +325,19 @@ let rec expression e =
       mcode print_string rp
   | Ast.TypeExp(ty) -> fullType ty
 
-  | Ast.MetaErr(name,_,_,_) -> 
+  | Ast.MetaErr(name,_,_,_) ->
       failwith "metaErr not handled"
 
   | Ast.MetaExpr (name,_,_,_typedontcare,_formdontcare,_) ->
       handle_metavar name  (function
-        | Ast_c.MetaExprVal exp -> 
+        | Ast_c.MetaExprVal exp ->
             pretty_print_c.Pretty_print_c.expression exp
         | _ -> raise Impossible
       )
 
-  | Ast.MetaExprList (name,_,_,_) -> 
+  | Ast.MetaExprList (name,_,_,_) ->
       handle_metavar name  (function
-        | Ast_c.MetaExprListVal args -> 
+        | Ast_c.MetaExprListVal args ->
             pretty_print_c.Pretty_print_c.arg_list args
         | _ -> raise Impossible
       )
@@ -355,7 +371,7 @@ let rec expression e =
       then mcode print_string dots
       else raise CantBeInPlus
 
-  | Ast.OptExp(exp) | Ast.UniqueExp(exp) -> 
+  | Ast.OptExp(exp) | Ast.UniqueExp(exp) ->
       raise CantBeInPlus
 
 and  unaryOp = function
@@ -452,9 +468,9 @@ and typeC ty =
       dots force_newline declaration decls;
       mcode print_string rb
   | Ast.TypeName(name)-> mcode print_string name
-  | Ast.MetaType(name,_,_) -> 
+  | Ast.MetaType(name,_,_) ->
       handle_metavar name  (function
-          Ast_c.MetaTypeVal exp -> 
+          Ast_c.MetaTypeVal exp ->
             pretty_print_c.Pretty_print_c.ty exp
         | _ -> raise Impossible)
 
@@ -534,6 +550,13 @@ and ft_space ty =
     Ast.Type(cv,ty) ->
       (match Ast.unwrap ty with
 	Ast.Pointer(_,_) -> ()
+      | Ast.MetaType(name,_,_) ->
+	  (match List.assoc (Ast.unwrap_mcode name) env with
+            Ast_c.MetaTypeVal (tq,ty) ->
+	      (match Ast_c.unwrap ty with
+		Ast_c.Pointer(_,_) -> ()
+	      |	_ -> pr_space())
+	  | _ -> pr_space())
       | _ -> pr_space())
   | _ -> pr_space()
 
@@ -561,7 +584,7 @@ and declaration d =
       mcode print_string sem
   | Ast.DisjDecl(_) | Ast.MetaDecl(_,_,_) -> raise CantBeInPlus
   | Ast.Ddots(_,_) -> raise CantBeInPlus
-  | Ast.OptDecl(decl)  | Ast.UniqueDecl(decl) -> 
+  | Ast.OptDecl(decl)  | Ast.UniqueDecl(decl) ->
       raise CantBeInPlus
 
 (* --------------------------------------------------------------------- *)
@@ -569,7 +592,7 @@ and declaration d =
 
 and initialiser nlcomma i =
   match Ast.unwrap i with
-    Ast.MetaInit(name,_,_) -> 
+    Ast.MetaInit(name,_,_) ->
       handle_metavar name  (function
           Ast_c.MetaInitVal ini ->
             pretty_print_c.Pretty_print_c.init ini
@@ -613,9 +636,9 @@ and parameterTypeDef p =
   | Ast.Param(ty,Some id) -> print_named_type ty id
   | Ast.Param(ty,None) -> fullType ty
 
-  | Ast.MetaParam(name,_,_) -> 
+  | Ast.MetaParam(name,_,_) ->
       failwith "not handling MetaParam"
-  | Ast.MetaParamList(name,_,_,_) -> 
+  | Ast.MetaParamList(name,_,_,_) ->
       failwith "not handling MetaParamList"
 
   | Ast.PComma(cm) -> mcode print_string cm
@@ -711,7 +734,7 @@ and rule_elem arity re =
   | Ast.Goto(goto,l,sem) ->
       mcode print_string goto; ident l; mcode print_string sem
   | Ast.Return(ret,sem) ->
-      pr_arity arity; mcode print_string ret; 
+      pr_arity arity; mcode print_string ret;
       mcode print_string sem
   | Ast.ReturnExpr(ret,exp,sem) ->
       pr_arity arity; mcode print_string ret; pr_space();
@@ -724,7 +747,7 @@ and rule_elem arity re =
   | Ast.Include(inc,s) ->
       mcode print_string inc; print_text " "; mcode inc_file s
   | Ast.DefineHeader(def,id,params) ->
-      mcode print_string def; pr_space(); ident id; 
+      mcode print_string def; pr_space(); ident id;
       print_define_parameters params
   | Ast.Default(def,colon) ->
       mcode print_string def; mcode print_string colon; pr_space()
@@ -780,7 +803,7 @@ let indent_if_needed s f =
     Ast.Seq(lbrace,body,rbrace) -> pr_space(); f()
   | _ ->
       (*no newline at the end - someone else will do that*)
-      start_block(); f(); unindent() in
+      start_block(); f(); unindent true in
 
 let rec statement arity s =
   match Ast.unwrap s with
@@ -862,7 +885,7 @@ let rec statement arity s =
 	 force_newline())
       else raise CantBeInPlus
 
-  | Ast.OptStm(s) | Ast.UniqueStm(s) -> 
+  | Ast.OptStm(s) | Ast.UniqueStm(s) ->
       raise CantBeInPlus
 
 and whencode notfn alwaysfn = function
@@ -897,7 +920,7 @@ let top_level t =
   | Ast.ERRORWORDS(exps) -> raise CantBeInPlus
 in
 
-(*    
+(*
 let rule =
   print_between (function _ -> force_newline(); force_newline()) top_level
 in
@@ -936,9 +959,11 @@ let rec pp_any = function
   | Ast.CaseLineTag(x) -> case_line "" x; false
 
   | Ast.ConstVolTag(x) -> const_vol x unknown unknown; false
-  | Ast.Pragma(xs) -> print_between force_newline print_text xs; false
+  | Ast.Pragma(xs) ->
+      let print = function Ast.Noindent s | Ast.Indent s -> print_text s in
+      print_between force_newline print xs; false
   | Ast.Token(x,None) -> print_text x; if_open_brace x
-  | Ast.Token(x,Some info) -> 
+  | Ast.Token(x,Some info) ->
       mcode
 	(fun x line lcol ->
 	  (match x with
@@ -951,7 +976,7 @@ let rec pp_any = function
 
   | Ast.Code(x) -> let _ = top_level x in false
 
-  (* this is not '...', but a list of expr/statement/params, and 
+  (* this is not '...', but a list of expr/statement/params, and
      normally there should be no '...' inside them *)
   | Ast.ExprDotsTag(x) -> dots (function _ -> ()) expression x; false
   | Ast.ParamDotsTag(x) -> parameter_list x; false
@@ -969,7 +994,7 @@ in
   (* todo? imitate what is in pretty_print_cocci ? *)
   match xxs with
     [] -> ()
-  | x::xs -> 
+  | x::xs ->
       (* for many tags, we must not do a newline before the first '+' *)
       let isfn s =
 	match Ast.unwrap s with Ast.FunDecl _ -> true | _ -> false in
@@ -978,7 +1003,7 @@ in
 	  (Ast.Token ("}",_)::_) -> true
 	| _ -> false in
       let prnl x =
-	(if unindent_before x then unindent());
+	(if unindent_before x then unindent true);
 	force_newline() in
       let newline_before _ =
 	if before =*= After
@@ -1015,7 +1040,7 @@ in
 	      match (indent_needed,unindent_before x) with
 		(true,true) -> force_newline()
 	      | (true,false) -> force_newline(); indent()
-	      | (false,true) -> unindent(); force_newline()
+	      | (false,true) -> unindent true; force_newline()
 	      | (false,false) -> force_newline());
 	    let space_needed_before = function
 		Ast.ParamTag(x) ->
