@@ -1,5 +1,5 @@
 (*
- * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
+ * Copyright 2005-2010, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
  * This file is part of Coccinelle.
  *
@@ -51,8 +51,8 @@ type info = { pos_info : position_info;
 	      attachable_start : bool; attachable_end : bool;
 	      mcode_start : mcodekind list; mcode_end : mcodekind list;
 	      (* the following are only for + code *)
-	      strings_before : (string * position_info) list;
-	      strings_after : (string * position_info) list }
+	      strings_before : (Ast.added_string * position_info) list;
+	      strings_after : (Ast.added_string * position_info) list }
 
 (* adjacency index is incremented when we skip over dots or nest delimiters
 it is used in deciding how much to remove, when two adjacent code tokens are
@@ -399,8 +399,8 @@ and parsed_rule =
 	(rule * Ast.metavar list) * Ast.ruletype
   | ScriptRule of
       string * Ast.dependency * (string * Ast.meta_name) list * string
-  | InitialScriptRule of string * string
-  | FinalScriptRule of string * string
+  | InitialScriptRule of string * Ast.dependency * string
+  | FinalScriptRule of string * Ast.dependency * string
 
 (* --------------------------------------------------------------------- *)
 
@@ -636,18 +636,22 @@ let rec reverse_type ty =
       if mv
       then
 	(* not right... *)
+	let rule = "" in
 	EnumName
 	  (make_mcode "enum",
-	   context_wrap(MetaId(make_mcode ("",tag),Ast.IdNoConstraint,Impure)))
+	   context_wrap(MetaId(make_mcode (rule,tag),Ast.IdNoConstraint,
+			       Impure)))
       else
 	EnumName(make_mcode "enum",context_wrap(Id(make_mcode tag)))
   | Type_cocci.StructUnionName(su,mv,tag) ->
       if mv
       then
 	(* not right... *)
+	let rule = "" in
 	StructUnionName
 	  (reverse_structUnion su,
-	   Some(context_wrap(MetaId(make_mcode ("",tag),Ast.IdNoConstraint,Impure))))
+	   Some(context_wrap(MetaId(make_mcode (rule,tag),Ast.IdNoConstraint,
+				    Impure))))
       else
 	StructUnionName
 	  (reverse_structUnion su,
