@@ -1,5 +1,5 @@
 (*
- * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
+ * Copyright 2005-2010, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
  * This file is part of Coccinelle.
  *
@@ -31,8 +31,8 @@ type incl_iso =
 
 type clt =
     line_type * int * int * int * int (* starting spaces *) *
-      (string * Ast0_cocci.position_info) list (* code before *) *
-      (string * Ast0_cocci.position_info) list (* code after *) *
+      (Ast_cocci.added_string * Ast0_cocci.position_info) list (*code before*) *
+      (Ast_cocci.added_string * Ast0_cocci.position_info) list (*code after *) *
       Ast0_cocci.meta_pos (* position variable, minus only *)
 
 (* ---------------------------------------------------------------------- *)
@@ -64,6 +64,11 @@ val clear_meta: (unit -> unit) ref
 val add_id_meta:
     (Ast_cocci.meta_name -> iconstraints -> Ast0_cocci.pure -> unit) ref
 
+val add_virt_id_meta_found: (string -> string -> unit) ref
+
+val add_virt_id_meta_not_found:
+    (Ast_cocci.meta_name -> Ast0_cocci.pure -> unit) ref
+
 val add_fresh_id_meta: (Ast_cocci.meta_name -> unit) ref
 
 val add_type_meta: (Ast_cocci.meta_name -> Ast0_cocci.pure -> unit) ref
