@@ -1,5 +1,5 @@
 (*
- * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
+ * Copyright 2005-2010, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
  * This file is part of Coccinelle.
  *
@@ -29,8 +29,3 @@ let show_SP = ref false
 let show_iso_failures = ref true
 
 let iso_limit = ref (None : int option) (*(Some 3)*)
-
-let defined_virtual_rules = ref ([] : string list)
-
-let set_defined_virtual_rules s =
-  defined_virtual_rules := s :: !defined_virtual_rules
