@@ -1,5 +1,5 @@
 (*
- * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
+ * Copyright 2005-2010, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
  * This file is part of Coccinelle.
  *
@@ -35,7 +35,6 @@ type typed_expinfo =
       Type_cocci.typeC list option * Data.clt
 type pos_info = Ast.meta_name * Data.pconstraints * Ast.meta_collect * Data.clt
 
-
 let get_option fn = function
     None -> None
   | Some x -> Some (fn x)
@@ -174,7 +173,7 @@ let ty_pointerify ty m =
 
 (* Left is <=>, Right is =>.  Collect <=>s. *)
 (* The parser should have done this, with precedences.  But whatever... *)
-let iso_adjust fn first rest =
+let iso_adjust first_fn fn first rest =
   let rec loop = function
       [] -> [[]]
     | (Common.Left x)::rest ->
@@ -186,7 +185,7 @@ let iso_adjust fn first rest =
 	  front::after -> []::(fn x::front)::after
 	| _ -> failwith "not possible") in
   match loop rest with
-    front::after -> (fn first::front)::after
+    front::after -> (first_fn first::front)::after
   | _ -> failwith "not possible"
 
 let check_meta tok =
@@ -346,6 +345,15 @@ let create_metadec ar ispure kindfn ids current_rule =
 	 kindfn ar rule ispure checker)
        ids)
 
+
+let create_metadec_virt ar ispure kindfn ids current_rule =
+  List.concat
+    (List.map
+       (function nm ->
+	 let checker = function x -> [Common.Right x] in
+	 kindfn ar nm ispure checker !Flag.defined_virtual_env)
+       ids)
+
 let create_fresh_metadec kindfn ids current_rule =
   List.concat
     (List.map
@@ -508,13 +516,13 @@ let make_script_rule_name_result lang deps =
   let l = id2name lang in
   Ast.ScriptRulename (l,deps)
 
-let make_initial_script_rule_name_result lang =
+let make_initial_script_rule_name_result lang deps =
   let l = id2name lang in
-  Ast.InitialScriptRulename(l)
+  Ast.InitialScriptRulename(l,deps)
 
-let make_final_script_rule_name_result lang =
+let make_final_script_rule_name_result lang deps =
   let l = id2name lang in
-  Ast.FinalScriptRulename(l)
+  Ast.FinalScriptRulename(l,deps)
 
 (* Allows type alone only when it is void and only when there is only one
     parameter.  This avoids ambiguity problems in the parser. *)
