@@ -1,5 +1,5 @@
 (*
- * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
+ * Copyright 2005-2010, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
  * This file is part of Coccinelle.
  *
@@ -112,7 +112,8 @@ let token2c (tok,_) =
   | PC.Tconst(clt) -> "const"^(line_type2c clt)
   | PC.Tvolatile(clt) -> "volatile"^(line_type2c clt)
 
-  | PC.TPragma(s,_) -> s
+  | PC.TPragma(Ast.Noindent s,_) -> s
+  | PC.TPragma(Ast.Indent s,_)   -> s
   | PC.TIncludeL(s,clt) -> (pr "#include \"%s\"" s)^(line_type2c clt)
   | PC.TIncludeNL(s,clt) -> (pr "#include <%s>" s)^(line_type2c clt)
   | PC.TDefine(clt,_) -> "#define"^(line_type2c clt)
@@ -266,6 +267,7 @@ let token2c (tok,_) =
   | PC.TIsoExpression -> "Expression"
   | PC.TIsoArgExpression -> "ArgExpression"
   | PC.TIsoTestExpression -> "TestExpression"
+  | PC.TIsoToTestExpression -> "ToTestExpression"
   | PC.TIsoStatement -> "Statement"
   | PC.TIsoDeclaration -> "Declaration"
   | PC.TIsoType -> "Type"
@@ -679,7 +681,8 @@ let split_token ((tok,_) as t) =
 
   | PC.TIso | PC.TRightIso
   | PC.TIsoExpression | PC.TIsoStatement | PC.TIsoDeclaration | PC.TIsoType
-  | PC.TIsoTopLevel | PC.TIsoArgExpression | PC.TIsoTestExpression ->
+  | PC.TIsoTopLevel | PC.TIsoArgExpression | PC.TIsoTestExpression
+  | PC.TIsoToTestExpression ->
       failwith "unexpected tokens"
   | PC.TScriptData s -> ([t],[t])
 
@@ -959,7 +962,7 @@ let check_parentheses tokens =
 	      (Printf.sprintf
 		 "unexpected close parenthesis in line %d\n" (clt2line clt))
 	| Common.Left _ :: seen_open -> loop seen_open rest
-	| Common.Right open_line :: _ -> 
+	| Common.Right open_line :: _ ->
 	    failwith
 	      (Printf.sprintf
 		 "disjunction parenthesis in line %d column 0 matched to normal parenthesis on line %d\n" open_line (clt2line clt)))
@@ -970,7 +973,7 @@ let check_parentheses tokens =
 	      (Printf.sprintf
 		 "unexpected close parenthesis in line %d\n" (clt2line clt))
 	| Common.Right _ :: seen_open -> loop seen_open rest
-	| Common.Left open_line :: _ -> 
+	| Common.Left open_line :: _ ->
 	    failwith
 	      (Printf.sprintf
 		 "normal parenthesis in line %d matched to disjunction parenthesis on line %d column 0\n" open_line (clt2line clt)))
@@ -1275,7 +1278,7 @@ let eval_virt virt =
       then
         failwith
           (Printf.sprintf "unknown virtual rule %s\n" x))
-    !Flag_parsing_cocci.defined_virtual_rules
+    !Flag.defined_virtual_rules
 
 let drop_last extra l = List.rev(extra@(List.tl(List.rev l)))
 
@@ -1334,8 +1337,8 @@ let get_rule_name parse_fn starts_with_name get_tokens file prefix =
       | Ast.GeneratedRulename (nm,a,b,c,d,e) ->
           Ast.GeneratedRulename (check_name nm,a,b,c,d,e)
       | Ast.ScriptRulename(s,deps) -> Ast.ScriptRulename(s,deps)
-      | Ast.InitialScriptRulename(s) -> Ast.InitialScriptRulename(s)
-      | Ast.FinalScriptRulename(s) -> Ast.FinalScriptRulename(s)
+      | Ast.InitialScriptRulename(s,deps) -> Ast.InitialScriptRulename(s,deps)
+      | Ast.FinalScriptRulename(s,deps) -> Ast.FinalScriptRulename(s,deps)
     else
       Ast.CocciRulename(Some(mknm()),Ast.NoDep,[],[],Ast.Undetermined,false) in
   Data.in_rule_name := false;
@@ -1373,7 +1376,7 @@ let parse_iso file =
 	    let (more,tokens) =
 	      get_tokens
 		[PC.TIsoStatement;PC.TIsoExpression;PC.TIsoArgExpression;
-		  PC.TIsoTestExpression;
+		  PC.TIsoTestExpression; PC.TIsoToTestExpression;
 		  PC.TIsoDeclaration;PC.TIsoType;PC.TIsoTopLevel] in
 	    let next_start = List.hd(List.rev tokens) in
 	    let dummy_info = ("",(-1,-1),(-1,-1)) in
@@ -1430,14 +1433,14 @@ let eval_depend dep virt =
       Ast.Dep req | Ast.EverDep req ->
 	if List.mem req virt
 	then
-	  if List.mem req !Flag_parsing_cocci.defined_virtual_rules
+	  if List.mem req !Flag.defined_virtual_rules
 	  then Some Ast.NoDep
 	  else None
 	else Some dep
     | Ast.AntiDep antireq | Ast.NeverDep antireq ->
 	if List.mem antireq virt
 	then
-	  if not(List.mem antireq !Flag_parsing_cocci.defined_virtual_rules)
+	  if not(List.mem antireq !Flag.defined_virtual_rules)
 	  then Some Ast.NoDep
 	  else None
 	else Some dep
@@ -1449,14 +1452,16 @@ let eval_depend dep virt =
     | Ast.OrDep(d1,d2) ->
 	(match (loop d1, loop d2) with
 	  (None,None) -> None
-	| (Some Ast.NoDep,x) | (x,Some Ast.NoDep) | (None,x) | (x,None) -> x
+	| (Some Ast.NoDep,x) | (x,Some Ast.NoDep) -> Some Ast.NoDep
+	| (None,x) | (x,None) -> x
 	| (Some x,Some y) -> Some (Ast.OrDep(x,y)))
     | Ast.NoDep | Ast.FailDep -> Some dep
     in
   loop dep
 
-let rec parse file =
+let parse file =
   Lexer_cocci.init();
+  let rec parse_loop file =
   let table = Common.full_charpos_to_pos file in
   Common.with_open_infile file (fun channel ->
   let lexbuf = Lexing.from_channel channel in
@@ -1490,7 +1495,7 @@ let rec parse file =
 	      |	(a,b,c)::rest ->
 		  let (x,y,z) = loop rest in
 		  (a::x,b::y,c::z) in
-	    loop (List.map parse include_files) in
+	    loop (List.map parse_loop include_files) in
 
           let parse_cocci_rule ruletype old_metas
 	      (rule_name, dependencies, iso, dropiso, exists, is_expression) =
@@ -1591,6 +1596,7 @@ let rec parse file =
 		  get_script_metavars PC.script_meta_main table file lexbuf) in
 
             let exists_in old_metas (py,(r,m)) =
+	      r = "virtual" or
               let test (rr,mr) x =
                 let (ro,vo) = Ast.get_meta_name x in
                 ro = rr && vo = mr in
@@ -1611,23 +1617,28 @@ let rec parse file =
             let data = collect_script_tokens tokens in
             (more,Ast0.ScriptRule(language, deps, metavars, data),[],tokens) in
 
-          let parse_if_script_rule k language =
+          let parse_if_script_rule k language _ deps =
             let get_tokens = tokens_script_all table file false lexbuf in
 
               (* script code *)
             let (more, tokens) = get_tokens [PC.TArobArob; PC.TArob] in
             let data = collect_script_tokens tokens in
-            (more,k (language, data),[],tokens) in
+            (more,k (language, deps, data),[],tokens) in
 
 	  let parse_iscript_rule =
 	    parse_if_script_rule
-	      (function (language,data) ->
-		Ast0.InitialScriptRule(language,data)) in
+	      (function (language,deps,data) ->
+		Ast0.InitialScriptRule(language,deps,data)) in
 
 	  let parse_fscript_rule =
 	    parse_if_script_rule
-	      (function (language,data) ->
-		Ast0.FinalScriptRule(language,data)) in
+	      (function (language,deps,data) ->
+		Ast0.FinalScriptRule(language,deps,data)) in
+
+	  let do_parse_script_rule fn l old_metas deps =
+	    match eval_depend deps virt with
+	      Some deps -> fn l old_metas deps
+	    | None ->  fn l old_metas Ast.FailDep in
 
           let parse_rule old_metas starts_with_name =
             let rulename =
@@ -1663,11 +1674,11 @@ let rec parse file =
 		    Data.in_generating := false;
 		    res)
             | Ast.ScriptRulename(l,deps) ->
-		(match eval_depend deps virt with
-		  Some deps -> parse_script_rule l old_metas deps
-		| None ->  parse_script_rule l old_metas Ast.FailDep)
-            | Ast.InitialScriptRulename(l) -> parse_iscript_rule l
-            | Ast.FinalScriptRulename(l)   -> parse_fscript_rule l
+		do_parse_script_rule parse_script_rule l old_metas deps
+            | Ast.InitialScriptRulename(l,deps) ->
+		do_parse_script_rule parse_iscript_rule l old_metas deps
+            | Ast.FinalScriptRulename(l,deps)   ->
+		do_parse_script_rule parse_fscript_rule l old_metas deps
             | _ -> failwith "Malformed rule name" in
 
 	  let rec loop old_metas starts_with_name =
@@ -1699,7 +1710,8 @@ let rec parse file =
   | (false,[(PC.TArobArob,_)]) | (false,[(PC.TArob,_)]) ->
       ([],([] : Ast0.parsed_rule list),[] (*virtual rules*))
   | _ -> failwith "unexpected code before the first rule\n" in
-  res)
+  res) in
+  parse_loop file
 
 (* parse to ast0 and then convert to ast *)
 let process file isofile verbose =
@@ -1716,8 +1728,8 @@ let process file isofile verbose =
     List.map
       (function
           Ast0.ScriptRule (a,b,c,d) -> [([],Ast.ScriptRule (a,b,c,d))]
-	| Ast0.InitialScriptRule (a,b) -> [([],Ast.InitialScriptRule (a,b))]
-	| Ast0.FinalScriptRule (a,b) -> [([],Ast.FinalScriptRule (a,b))]
+	| Ast0.InitialScriptRule (a,b,c) -> [([],Ast.InitialScriptRule (a,b,c))]
+	| Ast0.FinalScriptRule (a,b,c) -> [([],Ast.FinalScriptRule (a,b,c))]
 	| Ast0.CocciRule
 	    ((minus, metavarsm,
 	      (iso, dropiso, dependencies, rule_name, exists)),
@@ -1803,7 +1815,7 @@ let process file isofile verbose =
 	       let minus_ast =
 		 Ast0toast.ast0toast rule_name dependencies dropped_isos
 		   exists minus is_exp ruletype in
-	       
+
 	       match function_prototypes with
 		 None -> [(extra_meta @ metavars, minus_ast)]
 	       | Some mv_fp -> [(extra_meta @ metavars, minus_ast); mv_fp])
@@ -1817,8 +1829,8 @@ let process file isofile verbose =
   then List.iter Pretty_print_cocci.unparse code;
 
   let grep_tokens =
-    Common.profile_code "get_constants"
-      (fun () -> Get_constants.get_constants code) in (* for grep *)
+    Common.profile_code "get_constants" (* for grep *)
+      (fun () -> Get_constants.get_constants code) in
   let glimpse_tokens2 =
     Common.profile_code "get_glimpse_constants" (* for glimpse *)
       (fun () -> Get_constants2.get_constants code neg_pos) in
