@@ -1,5 +1,5 @@
 (*
- * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
+ * Copyright 2005-2010, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
  * This file is part of Coccinelle.
  *
@@ -40,7 +40,7 @@ let err wrapped ty s =
   T.typeC ty; Format.print_newline();
   failwith (Printf.sprintf "line %d: %s" (Ast0.get_line wrapped) s)
 
-type id = Id of string | Meta of (string * string)
+type id = Id of string | Meta of Ast.meta_name
 
 let int_type = T.BaseType(T.IntType)
 let bool_type = T.BaseType(T.BoolType)
@@ -189,15 +189,21 @@ let rec propagate_types env =
 		    Ast0.set_type exp1 ty; Ast0.set_type exp2 ty; ty in
 	      (match Ast0.unwrap_mcode op with
 		   Ast.Arith(op) -> same_type (ty1, ty2)
+		 | Ast.Logical(Ast.AndLog) | Ast.Logical(Ast.OrLog) ->
+		     Some(bool_type)
 		 | Ast.Logical(op) ->
 		     let ty = lub_type ty1 ty2 in
-		       Ast0.set_type exp1 ty; Ast0.set_type exp2 ty;
-		       Some(bool_type))
+		     Ast0.set_type exp1 ty; Ast0.set_type exp2 ty;
+		     Some(bool_type))
 	| Ast0.Paren(lp,exp,rp) -> Ast0.get_type exp
 	| Ast0.ArrayAccess(exp1,lb,exp2,rb) ->
 	    (match strip_cv (Ast0.get_type exp2) with
 		 None -> Ast0.set_type exp2 (Some(int_type))
 	       | Some(ty) when is_int_type ty -> ()
+	       | Some(Type_cocci.Unknown) ->
+		   (* unknown comes from param types, not sure why this
+		      is not just None... *)
+		   Ast0.set_type exp2 (Some(int_type))
 	       | Some ty -> err exp2 ty "bad type for an array index");
 	    (match strip_cv (Ast0.get_type exp1) with
 		 None -> None
@@ -337,7 +343,9 @@ let rec propagate_types env =
       (* if a type is known, it is specified in the decl *)
 	  None
       | (Ast0.Paren(lp,exp,rp),None) -> process_test exp
-      | (_,None) -> Some (int_type)
+      (* the following doesn't seem like a good idea - triggers int isos
+	 on all test expressions *)
+      (*| (_,None) -> Some (int_type) *)
       | _ -> None in
     let new_expty = process_test exp in
     (match new_expty with
