@@ -1,5 +1,5 @@
 (*
- * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
+ * Copyright 2005-2010, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
  * This file is part of Coccinelle.
  *
@@ -21,6 +21,7 @@
 
 
 open Format
+module Ast = Ast_cocci
 module Ast0 = Ast0_cocci
 module U = Pretty_print_cocci
 
@@ -51,7 +52,8 @@ let meta_pos = function
 (* Modified code *)
 
 let mcodekind brackets fn x info mc =
-  List.iter (function (s,_) -> print_string s) info.Ast0.strings_before;
+  let print = function Ast.Noindent s | Ast.Indent s -> print_string s in
+  List.iter (function (s,_) -> print s) info.Ast0.strings_before;
   (match mc with
     Ast0.MINUS(plus_stream) ->
       let (lb,rb) =
@@ -91,7 +93,7 @@ let mcodekind brackets fn x info mc =
       let (plus_streams,_,_) = !plus_streams in
       U.print_around (function x -> print_string lb; fn x; print_string rb)
 	x plus_streams);
-  List.iter (function (s,_) -> print_string s) info.Ast0.strings_after
+  List.iter (function (s,_) -> print s) info.Ast0.strings_after
 
 let mcode fn (x,_,info,mc,pos,adj) =
   let fn x = fn x; meta_pos !pos in
@@ -102,7 +104,8 @@ let print_context x fn =
     (Ast0.get_mcodekind x)
 
 let print_meta (ctx,name) =
-  if !full_ids then (print_string ctx; print_string ":");
+  (if !full_ids
+  then (print_string ctx; print_string ":"));
   print_string name
 
 (* --------------------------------------------------------------------- *)
