@@ -1,5 +1,5 @@
 (*
- * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
+ * Copyright 2005-2010, Ecole des Mines de Nantes, University of Copenhagen
  * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
  * This file is part of Coccinelle.
  *
@@ -46,7 +46,7 @@ type ('cd,'a) ccode = 'a combiner -> ('cd -> 'a) -> 'cd -> 'a
 
 val combiner :
     ('a -> 'a -> 'a) -> 'a ->
-      (((string*string),'a) cmcode) ->
+      ((Ast_cocci.meta_name,'a) cmcode) ->
       ((string,'a) cmcode) ->
       ((Ast_cocci.constant,'a) cmcode) ->
       ((Ast_cocci.assignOp,'a) cmcode) ->
@@ -103,7 +103,7 @@ type 'mc rmcode = 'mc Ast_cocci.mcode inout
 type 'cd rcode = rebuilder -> ('cd inout) -> 'cd inout
 
 val rebuilder :
-    ((string*string) rmcode) ->
+    (Ast_cocci.meta_name rmcode) ->
     (string rmcode) ->
     (Ast_cocci.constant rmcode) ->
     (Ast_cocci.assignOp rmcode) ->
